#!/usr/bin/perl

use strict;
use warnings;

my $infile = shift;

if (! $infile) {
    die "Usage: first_ts.pl 001.vdr";
}

open FH, $infile || die "$!\n";

my $buffer;
my $offset = 0;
my %pes_header;
my $rbytes = 0;
my $head_tmpl = 'H6 H2 n1 B8 B8 C1 B40';

$rbytes = sysread FH, $buffer, 100000, 0;

if (! $rbytes) {
    die "reading file $infile failed\n";

}

#print "buffer contains now " . length($buffer) . " bytes\n";

my $first_ts = 0;

while (! $first_ts) {

        ($pes_header{startcode}, $pes_header{stream_id}, 
        $pes_header{PES_packet_length}, $pes_header{eflags1}, 
        $pes_header{eflags2}, $pes_header{header_length},
        $pes_header{PTS_raw_bits}) 
        = unpack ( $head_tmpl, substr($buffer, $offset, 12) );
		
        $pes_header{payload_start} = (9 + $pes_header{header_length});
        # there are at leat six bytes of header at the beginning of a PES packet
        if ( $pes_header{startcode} ne "000001") {
            die "No 0x000001 found at current packet, searching for next Packet start\n";
        }
        # We check whether a TimeStamp is present:
        $pes_header{PTS_DTS_flags} = substr($pes_header{eflags2}, 0, 2);
        if (($pes_header{PTS_DTS_flags} eq "10") || ($pes_header{PTS_DTS_flags} eq "11")) {
            $pes_header{PTS_value_bits} = substr($pes_header{PTS_raw_bits},4,3) . substr($pes_header{PTS_raw_bits},8,15) . substr($pes_header{PTS_raw_bits},24,15);
            # decode the timestamp
            $pes_header{PTS_decimal} = unpack("N", (pack ("B32", substr($pes_header{PTS_value_bits},1))));
            $pes_header{PTS_decimal} += 4294967296 if (substr($pes_header{PTS_value_bits},0,1) == 1);
           
        }
        else {
            $pes_header{PTS_decimal} = 0;
            print "Found no TS\n";
        }
        if ($pes_header{PTS_decimal}) {
           $first_ts = $pes_header{PTS_decimal};
        }
        $offset +=  $pes_header{PES_packet_length} + 6;
}

print "FIRST PTS: $first_ts\n";
exit;






sub mark2frames{
	my $mark = shift;
	my($h, $m, $s, $f) = split /[:.]/, $mark;
	my $frame = (3600 * $h + 60 * $m + $s) * 25 + $f ;
	return $frame;
};

sub frames2mark {
	my $now = shift;
	print "now is $now\n";
	my $rest = $now  % 25;
	my $full_secs = int ($now  * 0.04);
	
	my $h = int($full_secs/3600);
	my $min = int(($full_secs - (3600 * $h)) / 60);
	my $sec = $full_secs - (3600 * $h + 60 * $min);
	return sprintf "%1u:%02u:%02u.%02u", $h, $min, $sec, $rest;
};

sub ticks2frames {
    my $ticks = shift;
    return $ticks / 3600;
}
